<?php

/**
 * Barcode edit form
 * @category    Insign
 * @package     Insign_Swisspost
 */
class Insign_Swisspost_Block_Adminhtml_Barcode_Edit_Form extends Mage_Adminhtml_Block_Widget_Form
{
    /**
     * prepare form
     * @access protected
     * @return Insign_Swisspost_Block_Adminhtml_Barcode_Edit_Form
     */
    protected function _prepareForm()
    {
        $config = $this->_getConfig();
        $shipmentId = $this->getRequest()->getParam('shipment_id');
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);

        $form = new Varien_Data_Form(
            array(
                'id'         => 'edit_form',
                'action'     => Mage::helper("adminhtml")->getUrl(
                    'swisspost/adminhtml_barcode/generate',
                    array(
                        'shipment_id' => $this->getRequest()->getParam('shipment_id')
                    )
                ),
                'method'     => 'post',
                'enctype'    => 'multipart/form-data'
            )
        );
        $form->setUseContainer(true);
        $form->setHtmlIdPrefix('barcode_');
        $form->setFieldNameSuffix('barcode');
        $this->setForm($form);
        $fieldset = $form->addFieldset(
            'barcode_form',
            array('legend' => Mage::helper('insign_swisspost')->__('Barcode'))
        );
        $fieldset->addType(
            'file',
            Mage::getConfig()->getBlockClassName('insign_swisspost/adminhtml_barcode_helper_file')
        );

        $fieldset->addField(
            'frankingLicense',
            'select',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Franking License'),
                'name'  => 'frankingLicense',
                'values'=> Mage::getModel('insign_swisspost/system_config_source_defaultLicense')->toOptionArray(),
                'value' => $config->getBarcodeFrankingLicense(),
                'required'  => true,
                'class' => 'required-entry',

            )
        );

        $fieldset->addField(
            'service_code',
            'select',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Service Code (DLC)'),
                'name'  => 'service_code',
                'value' => $this->getBarcodeServiceCode($shipment),
                'values'=> Mage::helper('insign_swisspost')->getDlcOptions(),
                'required'  => true,
                'class' => 'required-entry',

            )
        );
        $fieldset->addField(
            'additional_service',
            'multiselect',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Additional Service'),
                'name'  => 'additional_service',
                'value' => explode(',', $this->_getConfig()->getBarcodeAdditionalServices()),
                'values'=> Mage::getModel('insign_swisspost/system_config_source_additionalServices')->toOptionArray()
            )
        );


        $fieldset->addField(
            'name1',
            'text',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Sender Name'),
                'name'  => 'customer[name1]',
                'value' => $config->getBarcodeSenderName(),
                'required'  => true,
                'class' => 'required-entry',

            )
        );

        $fieldset->addField(
            'street',
            'text',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Street and House Number'),
                'name'  => 'customer[street]',
                'value' => $config->getBarcodeSenderAddress(),
                'required'  => true,
                'class' => 'required-entry',
            )
        );
        $fieldset->addField(
            'city',
            'text',
            array(
                'label' => Mage::helper('insign_swisspost')->__('City'),
                'name'  => 'customer[city]',
                'value' => $config->getBarcodeSenderCity(),
                'required'  => true,
                'class' => 'required-entry',

            )
        );
        $fieldset->addField(
            'zip',
            'text',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Zip'),
                'name'  => 'customer[zip]',
                'required'  => true,
                'value' => $config->getBarcodeSenderZip(),
                'class' => 'required-entry',

            )
        );
        $fieldset->addField(
            'domicilePostOffice',
            'text',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Domicil Post office'),
                'value' => $config->getBarcodeSenderDomicilePostOffice(),
                'name'  => 'customer[domicilePostOffice]'
            )
        );
        $fieldset->addField(
            'language',
            'select',
            array(
                'label'  => Mage::helper('insign_swisspost')->__('Language'),
                'name'   => 'language',
                'values' => [
                    [
                        'value' => 'DE',
                        'label' => 'DE',
                    ],
                    [
                        'value' => 'FR',
                        'label' => 'FR',
                    ],
                    [
                        'value' => 'IT',
                        'label' => 'IT',
                    ],
                    [
                        'value' => 'EN',
                        'label' => 'EN',
                    ]
                ]
            )
        );
        $fieldset->addField(
            'imageFileType',
            'select',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Image File Type'),
                'name'  => 'labelDefinition[imageFileType]',
                'value' => $config->getBarcodeLabelImageFiletype(),
                'values'=> Mage::getModel('insign_swisspost/system_config_source_imageFiletype')->toOptionArray()
            )
        );
        $fieldset->addField(
            'imageResolution',
            'select',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Image Resolution'),
                'name'  => 'labelDefinition[imageResolution]',
                'value' => $config->getBarcodeLabelImageResolution(),
                'values'=> Mage::getModel('insign_swisspost/system_config_source_imageResolution')->toOptionArray()
            )
        );
        $fieldset->addField(
            'labelLayout',
            'select',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Label Layout'),
                'name'  => 'labelDefinition[labelLayout]',
                'value' => $config->getBarcodeLabelLayout(),
                'required'  => true,
                'class' => 'required-entry',
                'values'=> Mage::getModel('insign_swisspost/system_config_source_labelLayout')->toOptionArray()
            )
        );
        $fieldset->addField(
            'printAddresses',
            'select',
            array(
                'label' => Mage::helper('insign_swisspost')->__('Print Addresses'),
                'name'  => 'labelDefinition[printAddresses]',
                'value' => $config->getBarcodeLabelPrintAddresses(),
                'values'=> Mage::getModel('insign_swisspost/system_config_source_printAddresses')->toOptionArray()
            )
        );
        return parent::_prepareForm();
    }

    /**
     * @return Insign_Swisspost_Model_Config
     */
    private function _getConfig()
    {
        return Mage::getModel('insign_swisspost/config');
    }

    /**
     * @param $shipment
     * @return string
     */
    public function getBarcodeServiceCode($shipment)
    {
        $shippingMethod = $shipment->getOrder()->getShippingMethod(true)->getMethod();
        $basicServices = $this->_getConfig()->getBarcodeBasicServices($shippingMethod);
        return  is_array($basicServices) ? implode(',', $basicServices) : '';

    }
}
