<?php

class Insign_Swisspost_Model_AddressValidator
{
    /**
     * @param $address
     * @return array
     */
    private function _formatAddress($address)
    {
        $address = $this->_streetFormat($address);
        return [
            'addressee' => [
                'firstName' => $address['firstname'] ?? null,
                'lastName'  =>  $address['lastname'] ?? null,
            ],
            'geographicLocation' => [
                'house' => [
                    'street' =>  $address['street'] ?? null,
                    'houseNumber' =>  $address['nr'] ?? null,
                    'additionalAddress' => ''
                ],
                'zip' => [
                    'zip' =>  $address['postcode'] ?? null,
                    'city' =>  $address['city'] ?? null,
                ]
            ],
            'fullValidation' => true
        ];
    }

    /**
     * @param $address
     * @return mixed
     */
    private function _streetFormat($address)
    {
        if(isset($address['street'])) {
            $street = $address['street'];
            $streetName = is_array($street) ? $street[0] : $street;
            $splitStreet = $this->_getStreetSection($streetName);
            unset($address['street']);
            $address['street'] = $splitStreet[1]?? null;
            $address['nr'] = $splitStreet[2]?? null;
        }
        return $address;
    }

    /**
     * @param $v
     * @return mixed
     */
    private function _getStreetSection($v) {
        $c = 'äáàâåëéèêïíìîöóòôüúùûýÿ'
            . 'ÄÁÀÂËÉÈÊÏÍÌÎÖÓÒÔÜÚÙÛÝß'
            . 'ñÑÇçšæÆ';
        $r = "/^(\d*[\w{$c}\d '\/\-\.]+)[,\s]+(\d+)\s*([\w{$c}\d\-\/]*)$/i";
        preg_match( $r, $v, $m );
        return $m;
    }

    /**
     * @param $address
     * @return bool|mixed
     * @throws Mage_Core_Exception
     * @throws Zend_Http_Client_Exception
     * @throws Zend_Json_Exception
     */
    public function validate($address)
    {
        $rest = Mage::getSingleton('insign_swisspost/rest');
        $config = $rest->getConfig();
        if(!$config->isAddressValidatorEnabled()) {
            return true;
        }
        $endPointUrl = $config->getAddressValidatorEndPoint();
        $options = $this->_formatAddress($address);
        try {
            $data = $rest->getClient()->authenticatedRequest(Zend_Http_Client::POST, $endPointUrl, $options);
            $response =  Zend_Json::decode($data->getBody());
        } catch (\Exception $e) {
            throw  $e;
        }
        if($response && is_array($response) && $response['quality']) {
            $quality = $response['quality'];
            return $quality;
        }
        return false;
    }
}
