<?php

class Insign_Swisspost_Model_BarcodeProvider
{
    /**
     * @param $shipment Mage_Sales_Model_Order_Shipment
     * @throws Mage_Core_Exception
     */
    public function createBarcode($shipment, $priorityBarcodeOptions = [])
    {
        $rest = Mage::getSingleton('insign_swisspost/rest');
        $storeId = $shipment->getOrder()->getStore()->getId();
        /** @var  Insign_Swisspost_Model_Config $config */
        $config = $rest->getConfig()->setStoreId($storeId);
        $endPointUrl = $config->getBarcodeEndpoint();
        $tokenScope = $config->getBarcodeScope();
        $shippingAddress = $shipment->getShippingAddress();
        $street = $shippingAddress->getStreet();
        $street = $config->getStreetSection($street[0]??null);
        $streetName = $street[1]?? null;
        $streetNumber = $street[2]?? null;
        $code = substr(Mage::getStoreConfig('general/locale/code', $storeId),0,2);
        $code = strtoupper($code) ?? 'DE';
        $code =  in_array($code, [ 'DE', 'FR', 'IT', 'EN' ]) ? $code : 'DE';
        $company = $shippingAddress->getCompany();
        $customerName = $shippingAddress->getFirstname().' '.$shippingAddress->getLastname();
        $name1 = ($company) ? $company : $customerName;
        $name2 = ($company) ? $customerName : false;
        $barcodeOptions = [
            'language' => $code,
            'frankingLicense' => $config->getBarcodeFrankingLicense(),
            'customer' => [
                'name1' => $config->getBarcodeSenderName(),
                'street' => $config->getBarcodeSenderAddress(),
                'zip' => $config->getBarcodeSenderZip(),
                'city' => $config->getBarcodeSenderCity(),
                'domicilePostOffice' => $config->getBarcodeSenderDomicilePostOffice(),
            ],
            'labelDefinition' => [
                'labelLayout' => $config->getBarcodeLabelLayout(),
                'printAddresses' => $config->getBarcodeLabelPrintAddresses(),
                'imageFileType' => $config->getBarcodeLabelImageFiletype(),
                'imageResolution' => $config->getBarcodeLabelImageResolution(),
            ],
            'item' => [
                'itemID' => $this->getItemId(),
                'recipient' => [
                    'name1' => $name1,
                    'street' => $streetName,
                    'houseNo' => $streetNumber,
                    'zip' => $shippingAddress->getPostcode(),
                    'city' => $shippingAddress->getCity(),
                    'country' => $shippingAddress->getCountryId(),
                ],
                'attributes' => [
                    'przl' => $this->getServiceCode($config, $shipment)
                ],
            ]
        ];
        if($name2) { // remove if
            $barcodeOptions['item']['recipient']['name2'] = $name2;
        }
        $logo = $config->getBarcodeLogoFile();
        if($logo) {
            $logoData = [
                'logo' => base64_encode(file_get_contents($logo)),
                'logoFormat' => strtoupper(pathinfo($logo, PATHINFO_EXTENSION)),
                'logoRotation' => $config->getBarcodeLogoRotation(),
                'logoAspectRatio' => $config->getBarcodeLogoAspectRatio(),
                'logoHorizontalAlign' => $config->getBarcodeLogoHorizontalAlign(),
                'logoVerticalAlign' => $config->getBarcodeLogoVerticalAlign()
            ];
            $barcodeOptions['customer'] = array_merge($barcodeOptions['customer'], $logoData);
        }

        if(!empty($priorityBarcodeOptions)) {
            if(isset($priorityBarcodeOptions['item']['attributes']['przl'])) {
                $barcodeOptions['item']['attributes']['przl'] = $priorityBarcodeOptions['item']['attributes']['przl'];
                unset($priorityBarcodeOptions['item']['attributes']['przl']);
            }
            $barcodeOptions = array_replace_recursive($barcodeOptions, $priorityBarcodeOptions);
        }
        try {
            $data = $rest->getClient()->authenticatedRequest(Zend_Http_Client::POST, $endPointUrl, $barcodeOptions, $tokenScope);
            $response = Zend_Json::decode($data->getBody());
        } catch (Exception $e) {
            throw $e;
        }
        $this->processBarcode($response, $shipment, $config);
        return $this;

    }


    /**
     * @param $response array
     * @param $shipment Mage_Sales_Model_Order_Shipment
     * @param $config Insign_Swisspost_Model_Config
     * @return false
     * @throws Mage_Core_Exception
     */
    private function processBarcode($response, $shipment, $config)
    {
        $type = $response['labelDefinition']['imageFileType'] ?? null;
        $fileContent = $response['item']['label'][0] ?? null;
        $identityCode = $response['item']['identCode'] ?? null;
        $errors = $response['item']['errors'] ?? null;
        if(is_array($errors)) {
            $errorMsg = '';
            foreach ($errors as $key => $value) {
                $code = $value['code'] ?? '';
                $msg =  $value['message'] ?? ' ';
                $errorMsg .= $code.'=>'.$msg.' '.PHP_EOL;
            }
            throw new Mage_Core_Exception($errorMsg, 0);

        }
        if(!$type || !$fileContent) {
            throw new Mage_Core_Exception('There is somme error while generaing barcode', 0);
        }
        try {
            $filename = $config->storeBarcodeFile($type, $fileContent);
        } catch (Mage_Core_Exception $e) {
            throw $e;
        }
        $barcode = Mage::getModel('insign_swisspost/barcode');
        $barcode->setShipmentId($shipment->getId())
            ->setLabelFilePath($filename)
            ->setIdentityCode($identityCode)
            ->save();
    }

    /**
     * @return int
     */
    private function getItemId()
    {
        return Mage::getResourceModel('insign_swisspost/barcode_collection')->getLastItemId();

    }

    /**
     * @param $config Insign_Swisspost_Model_Config
     * @param $shipment Mage_Sales_Model_Order_Shipment
     */
    private function getServiceCode($config, $shipment)
    {
        $shippingMethod = $shipment->getOrder()->getShippingMethod(true)->getMethod();
        $przl = $config->getBarcodeBasicServices($shippingMethod);
        $additionalServiceCode = $config->getBarcodeAdditionalServices();
        if(empty($przl)) {
            $przl = ['ECO'];
        }
        if($additionalServiceCode && $additionalServiceCode !== '') {
            $additionalServiceCode = explode(',', $additionalServiceCode);
            $przl = array_merge($przl, $additionalServiceCode);
        }
        return $przl;
    }
}
