<?php

class Insign_Swisspost_Model_Config
{
    const XML_PATH_ENABLED = 'insign_swisspost/settings/enabled';
    const XML_PATH_TOKEN_ENDPOINT_URL = 'insign_swisspost/settings/url';
    const XML_PATH_CLIENT_ID = 'insign_swisspost/settings/client_id';
    const XML_PATH_CLIENT_SECRET = 'insign_swisspost/settings/client_secret';
    const XML_PATH_ENABLE_DEBUG = 'insign_swisspost/settings/enable_debug';

    const XML_PATH_ADDRESS_VALIDATOR_ENABLE = 'insign_swisspost/validator/enabled';
    const XML_PATH_ADDRESS_VALIDATOR_SCOPE = 'insign_swisspost/validator/scope';
    const XML_PATH_ADDRESS_VALIDATOR_ENDPOINT = 'insign_swisspost/validator/validator_endpoint';

    const XML_PATH_ADDRESS_AUTOCOMPLETE_ENABLE = 'insign_swisspost/autocomplete/enabled';
    const XML_PATH_ADDRESS_AUTOCOMPLETE_SCOPE = 'insign_swisspost/autocomplete/scope';
    const XML_PATH_ADDRESS_AUTOCOMPLETE_ENDPOINT_ZIP = 'insign_swisspost/autocomplete/autocomplete_endpoint_zip';
    const XML_PATH_ADDRESS_AUTOCOMPLETE_ENDPOINT_STREET = 'insign_swisspost/autocomplete/autocomplete_endpoint_street';

    const XML_PATH_BARCODE_GENERATION_ENABLED = 'insign_swisspost/barcode_generation/enabled';
    const XML_PATH_BARCODE_GENERATION_SCOPE = 'insign_swisspost/barcode_generation/scope';
    const XML_PATH_BARCODE_GENERATION_ENDPOINT = 'insign_swisspost/barcode_generation/endpoint';
    const XML_PATH_BARCODE_GENERATION_FRANKING_LICENSE_PROVIDER = 'insign_swisspost/barcode_generation/franking_license_provider';
    const XML_PATH_BARCODE_GENERATION_FRANKING_LICENSE = 'insign_swisspost/barcode_generation/franking_license';
    const XML_PATH_BARCODE_GENERATION_BASIC_SERVICES = 'insign_swisspost/barcode_generation/basic_services';
    const XML_PATH_BARCODE_GENERATION_ADDITIONAL_SERVICES = 'insign_swisspost/barcode_generation/additional_services';

    const XML_PATH_BARCODE_LABEL_LAYOUT = 'insign_swisspost/barcode_label/layout';
    const XML_PATH_BARCODE_LABEL_PRINT_ADDRESSES = 'insign_swisspost/barcode_label/print_addresses';
    const XML_PATH_BARCODE_LABEL_IMAGE_RESOLUTION = 'insign_swisspost/barcode_label/image_resolution';
    const XML_PATH_BARCODE_LABEL_IMAGE_FILETYPE = 'insign_swisspost/barcode_label/image_filetype';

    const XML_PATH_BARCODE_LOGO_FILE = 'insign_swisspost/barcode_logo/file';
    const XML_PATH_BARCODE_LOGO_ROTATION = 'insign_swisspost/barcode_logo/rotation';
    const XML_PATH_BARCODE_LOGO_ASPECT_RATIO = 'insign_swisspost/barcode_logo/aspect_ratio';
    const XML_PATH_BARCODE_LOGO_VERTICAL_ALIGN = 'insign_swisspost/barcode_logo/vertical_align';
    const XML_PATH_BARCODE_LOGO_HORIZONTAL_ALIGN = 'insign_swisspost/barcode_logo/horizontal_align';

    const XML_PATH_BARCODE_SENDER_CUSTOM_ENABLE = 'insign_swisspost/barcode_sender/enable_custom_sender';
    const XML_PATH_BARCODE_SENDER_NAME = 'insign_swisspost/barcode_sender/name';
    const XML_PATH_BARCODE_SENDER_ADDRESS = 'insign_swisspost/barcode_sender/address';
    const XML_PATH_BARCODE_SENDER_ZIP = 'insign_swisspost/barcode_sender/zip';
    const XML_PATH_BARCODE_SENDER_CITY = 'insign_swisspost/barcode_sender/city';
    const XML_PATH_BARCODE_SENDER_DOMICILE_POST_OFFICE = 'insign_swisspost/barcode_sender/domicile_post_office';

    const BARCODE_DIR = 'swisspost/barcode/codes';
    const BARCODE_LOGO_DIR = 'catalog/product/placeholder';
    private $_storeId = null;

    /**
     * @param null $storeId
     * @return $this
     */
    public function setStoreId($storeId = null)
    {
        $this->_storeId = $storeId;
        return $this;
    }

    private function _getConfig(string $path)
    {
        return Mage::getStoreConfig($path, $this->_storeId);
    }

    public function isDebugModeEnabled()
    {
        return Mage::getStoreConfigFlag(self::XML_PATH_ENABLE_DEBUG);
    }
    public function isEnabled($storeId = null)
    {
        if (Mage::getStoreConfigFlag(self::XML_PATH_ENABLED) &&
            $this->getTokenUrl() &&
            $this->getClientId() &&
            $this->getClientSecret()) {
            return true;
        }
        return false;
    }

    public function getClientId()
    {
        return trim($this->_getConfig(self::XML_PATH_CLIENT_ID));
    }

    public function getClientSecret()
    {
        return trim($this->_getConfig(self::XML_PATH_CLIENT_SECRET));
    }

    public function isAddressValidatorEnabled()
    {
        if($this->isEnabled()) {
            if($this->_getConfig(self::XML_PATH_ADDRESS_VALIDATOR_ENABLE)) {
                return true;
            }
        }
        return false;
    }
    public function getAddressValidatorEndPoint()
    {
        return trim($this->_getConfig(self::XML_PATH_ADDRESS_VALIDATOR_ENDPOINT));
    }

    public function getTokenUrl()
    {
        $endPointUrl =  trim($this->_getConfig(self::XML_PATH_TOKEN_ENDPOINT_URL));
        $endPointUrl = rtrim($endPointUrl, '/') . '/';
        return $endPointUrl;
    }
    public function getValidatorScope()
    {
        return trim($this->_getConfig(self::XML_PATH_ADDRESS_VALIDATOR_SCOPE));
    }
    /** Autocomplete Config */
    public function isAddressAutocompleteEnabled()
    {
        if($this->isEnabled()) {
            if(Mage::getStoreConfigFlag(self::XML_PATH_ADDRESS_AUTOCOMPLETE_ENABLE)) {
                return true;
            }
        }
        return false;
    }
    public function getAddressAutocompleteZipEndpoint()
    {
        return trim($this->_getConfig(self::XML_PATH_ADDRESS_AUTOCOMPLETE_ENDPOINT_ZIP));
    }
    public function getAddressAutocompleteStreetEndpoint()
    {
        return trim($this->_getConfig(self::XML_PATH_ADDRESS_AUTOCOMPLETE_ENDPOINT_STREET));
    }
    public function getAutocompleteScope()
    {
        return trim($this->_getConfig(self::XML_PATH_ADDRESS_AUTOCOMPLETE_SCOPE));
    }

    /** Barcode Config */
    public function isBarcodeGenerationEnabled()
    {
        if($this->isEnabled()) {
            if(Mage::getStoreConfigFlag(self::XML_PATH_BARCODE_GENERATION_ENABLED)) {
                return true;
            }
        }
        return false;
    }
    public function getBarcodeEndpoint()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_GENERATION_ENDPOINT));
    }
    public function getBarcodeScope()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_GENERATION_SCOPE));
    }
    public function getBarcodeFrankingLicense()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_GENERATION_FRANKING_LICENSE));
    }

    public function getBarcodeLicenseProvider($withExtraLabel = false)
    {
        $data = trim($this->_getConfig(self::XML_PATH_BARCODE_GENERATION_FRANKING_LICENSE_PROVIDER));
        $data = unserialize($data);
        $codes = [];
        foreach ($data as $row) {
            if(!isset($row['label']) || !isset($row['value'])) {
                continue;
            }
            $row['label'] = ($withExtraLabel) ? $row['label']. '('.$row['value'].')' : $row['label'];
            $codes[] = $row;
        }
       return $codes;

    }
    public function getBarcodeLabelLayout()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LABEL_LAYOUT));
    }
    public function getBarcodeLabelPrintAddresses()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LABEL_PRINT_ADDRESSES));
    }
    public function getBarcodeLabelImageResolution()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LABEL_IMAGE_RESOLUTION));
    }
    public function getBarcodeLabelImageFiletype()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LABEL_IMAGE_FILETYPE));
    }

    public function getBarcodeLogoFile()
    {
        $logo =  trim($this->_getConfig(self::XML_PATH_BARCODE_LOGO_FILE));
        $logoWithPath = Mage::getBaseDir('media') . DS .self::BARCODE_LOGO_DIR. DS.$logo;
        if($logo && file_exists($logoWithPath)) {
            return $logoWithPath;
        }
        return false;
    }
    public function getBarcodeLogoRotation()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LOGO_ROTATION));
    }
    public function getBarcodeLogoAspectRatio()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LOGO_ASPECT_RATIO));
    }
    public function getBarcodeLogoVerticalAlign()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LOGO_VERTICAL_ALIGN));
    }
    public function getBarcodeLogoHorizontalAlign()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_LOGO_HORIZONTAL_ALIGN));
    }

    public function getBarcodeBasicServices($shippingMethod = '')
    {
        $data = trim($this->_getConfig(self::XML_PATH_BARCODE_GENERATION_BASIC_SERVICES));
        $data = unserialize($data);
        $codes = [];
        foreach ($data as $row) {
            if(isset($row['shipping_method'])) {
                $codes[$row['shipping_method']] = $row['swiss_post_dlc'] ?? '';
            }
        }
        if($shippingMethod) {
            return [$codes[$shippingMethod]] ?? [];
        }
        return array_values($codes);
    }
    public function getBarcodeAdditionalServices()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_GENERATION_ADDITIONAL_SERVICES));
    }


    public function isBarcodeCustomSenderEnabled()
    {
        return $this->_getConfig(self::XML_PATH_BARCODE_SENDER_CUSTOM_ENABLE);
    }
    public function getBarcodeSenderName()
    {
        if(!$this->isBarcodeCustomSenderEnabled()) {
            return trim($this->_getConfig('general/store_information/name'));
        }
        return trim($this->_getConfig(self::XML_PATH_BARCODE_SENDER_NAME));
    }
    public function getBarcodeSenderAddress()
    {
        if(!$this->isBarcodeCustomSenderEnabled()) {
            return trim($this->_getConfig('general/store_information/street_line1'));
        }
        return trim($this->_getConfig(self::XML_PATH_BARCODE_SENDER_ADDRESS));
    }
    public function getBarcodeSenderZip()
    {
        if(!$this->isBarcodeCustomSenderEnabled()) {
            return trim($this->_getConfig('general/store_information/postcode'));
        }
        return trim($this->_getConfig(self::XML_PATH_BARCODE_SENDER_ZIP));
    }
    public function getBarcodeSenderCity()
    {
        if(!$this->isBarcodeCustomSenderEnabled()) {
            return trim($this->_getConfig('general/store_information/city'));
        }
        return trim($this->_getConfig(self::XML_PATH_BARCODE_SENDER_CITY));
    }

    public function getBarcodeSenderDomicilePostOffice()
    {
        return trim($this->_getConfig(self::XML_PATH_BARCODE_SENDER_DOMICILE_POST_OFFICE));
    }

    /**
     * @param string $type
     * @param string $imageData
     * @return string
     * @throws Mage_Core_Exception
     */
    public function storeBarcodeFile(string $type, string $imageData)
    {
        $filename = md5(microtime()) . '.'.$type;
        $io = new Varien_Io_File();
        $path = Mage::getBaseDir('var') . DS .self::BARCODE_DIR. DS.$filename;
        try {
            $io->mkdir(dirname($path), 0775);
            $io->filePutContent($path, base64_decode($imageData));
        } catch (Exception $e) {
            throw new Mage_Core_Exception($e->getMessage(), 0);
        }
        return $filename;
    }
    public function getFullBarcodePathByFileName(string $fileName)
    {
        $filenameWithPath = Mage::getBaseDir('var') . DS .self::BARCODE_DIR. DS .$fileName;
        if(!file_exists($filenameWithPath)) {
            return false;
        }
        return $filenameWithPath;
    }

    /**
     * @param $input
     * @param $data
     * @return false|string
     * @throws Exception
     */
    public function uploadAndGetName($input, $data)
    {
        $path = Mage::getBaseDir('media') . DS .self::BARCODE_LOGO_DIR. DS;
        try {
            if (isset($data[$input]['delete'])) {
                return false;
            } else {
                $uploader = new Varien_File_Uploader($input);
                $uploader->setAllowedExtensions(array('png', 'jpg','gif'));
                $uploader->setAllowRenameFiles(true);
                $uploader->setFilesDispersion(true);
                $uploader->setAllowCreateFolders(true);
                $result = $uploader->save($path);
                return $path.$result['file'];
            }
        } catch (Exception $e) {
            if ($e->getCode() != Varien_File_Uploader::TMP_NAME_EMPTY) {
                throw $e;
            }
        }
        return false;
    }

    /**
     * separate street and house number including German special character. $m[1] => streetname, $m[2] => house no.
     * @param $street
     * @return mixed
     */
    public function getStreetSection($street)
    {
        $c = 'äáàâåëéèêïíìîöóòôüúùûýÿ'
            . 'ÄÁÀÂËÉÈÊÏÍÌÎÖÓÒÔÜÚÙÛÝß'
            . 'ñÑÇçšæÆ';
        $r = "/^(\d*[\w{$c}\d '\/\-\.]+)[,\s]+(\d+)\s*([\w{$c}\d\-\/]*)$/i";
        preg_match( $r, $street, $m );
        return $m;
    }
    public function log($msg, $level = Zend_Log::DEBUG, $forceLog = true)
    {
        Mage::log($msg, $level, 'swisspost.log', $forceLog);
    }

    /**
     * @param $request
     * @param $response Zend_Http_Response
     * @param string $endpointUrl
     */
    public function debugLog($endpointUrl, $request, $response)
    {
        if($this->isDebugModeEnabled()) {
            try {
                $request = is_array($request) ? json_encode($request) : $request;
                $response = $response->asString();
                $msg = 'ENDPOINT_URL: '.PHP_EOL.$endpointUrl.PHP_EOL.PHP_EOL;
                $msg .= 'REQUEST:'.PHP_EOL.$request.PHP_EOL.PHP_EOL;
                $msg .= 'RESPONSE: '.PHP_EOL.$response;;
                $this->log($msg);
            } catch (Exception $e) {
                $this->log($e->getMessage());
            }
        }
    }
}
