<?php

class Insign_Swisspost_Model_Oauth
{
    /** @var Insign_Swisspost_Model_Config */
    protected $_config;
    protected $_accessToken;
    private $requestCounter = 0;
    private $_tokenScope = 'WEDEC_VALIDATE_ADDRESS';
    /**
     * @var Varien_Http_Client
     */
    private  $_httpClient;

    public function __construct()
    {
        $args = func_get_args();
        if(isset($args[0]) && is_array($args[0])) {
            $args = $args[0];
        }
        if(count($args)<1) {
            trigger_error('Missing arguments for REST',E_USER_ERROR);
        }
        $this->_config      = $args[0];
    }
    /**
     * @return Varien_Http_Client
     */
    private function _getHttpClient()
    {
        if (!$this->_httpClient instanceof Varien_Http_Client) {
            $this->_httpClient = new Varien_Http_Client();
        }
        return $this->_httpClient;
    }
    private function _prepareParams($params)
    {
        foreach ($params as $key => &$val) {
            if (!is_array($val)) continue;
            $val = Zend_Json::encode($val);
        }
        $rawData = '';
        foreach ($params as $postParam => $postValue) {
            if ($rawData !== '') {
                $rawData .= '&';
            }
            $rawData .= $postParam . '=' . $postValue;
        }
        return $rawData;
    }

    private function _formatUrl($url)
    {
        $url = rtrim($url, '/') . '/';
        return $url;

    }
    private function _createFreshToken()
    {
        $params = $this->_prepareParams(array(
            'grant_type'		=>	'client_credentials',
            'client_id'		=>	$this->_config->getClientId(),
            'client_secret'	=>	$this->_config->getClientSecret(),
            'scope' => $this->_tokenScope,
        ));
        try {
            $accessTokenResponse = $this->_getHttpClient()
                ->setUri($this->_formatUrl($this->_config->getTokenUrl()))
                ->setMethod(Zend_Http_Client::POST)
                ->resetParameters()
                ->setRawData($params)
                ->request()
                ->getBody();
            $responseParams= Zend_Json::decode($accessTokenResponse);
            if (isset($responseParams['access_token'])) {
                $this->_accessToken = $responseParams['access_token'];
            }
        } catch(Exception $e) {
            throw new Mage_Core_Exception($e->getMessage(), 0);
        }
        $expireInSeconds = (int) $responseParams['expires_in'];
        $this->_persistTokenAccessDb($expireInSeconds);
        return $this->_accessToken;
    }

    private function _persistTokenAccessDb($expireInSeconds)
    {
        $date = new DateTime();
        $date->add(new DateInterval('PT'.$expireInSeconds.'S'));
        $expireIn = $date->getTimestamp(); // adds 674165 secs
        $tokenModel = Mage::getModel('insign_swisspost/token')->getCollection()->getAccessTokenByScope($this->_tokenScope);
        $tokenModel = ($tokenModel) ?: Mage::getModel('insign_swisspost/token');
        try {
            $tokenModel->setScope($this->_tokenScope)
                ->setToken($this->_accessToken)
                ->setExpireIn($expireIn)
                ->save();
        } catch (Exception $e) {
        }
    }

    /**
     * @return mixed
     */
    private function _getDbAccessToken()
    {
        return Mage::getModel('insign_swisspost/token')->getCollection()->getAccessToken($this->_tokenScope);
    }
    /**
     * @return mixed
     * @throws Mage_Core_Exception
     */
    public function getAccessToken()
    {
        if($this->_accessToken) {
            return $this->_accessToken;
        }
        if($accessToken = $this->_getDbAccessToken()) {
            return $accessToken;
        }
        return $this->_createFreshToken();
    }

    /**
     * @return string[]
     * @throws Mage_Core_Exception
     */
    private function _getAuthorisedHeader()
    {
        return ['Authorization' => 'Bearer ' . $this->getAccessToken()];
    }

    /**
     * @param $url
     * @param $data
     * @param string $method
     * @return mixed|null
     * @throws Mage_Core_Exception
     * @throws Zend_Http_Client_Exception
     * @throws Zend_Json_Exception
     */
    private function _oauthRequest($url, $data, string $method)
    {
        $client = $this->_getHttpClient()
            ->setUri($url)
            ->setHeaders($this->_getAuthorisedHeader())
            ->setMethod($method)
            ->resetParameters()
            ->setParameterPost($data);
        try {
            $response = $client->request();
        } catch(Exception $e) {
            throw new Mage_Core_Exception('Service temporarily unavailable.');
        }
        return Zend_Json::decode($response->getBody());
    }

    /** This is the main access point for any OAUTH request.
     * @param $url
     * @param $options
     * @param string $method
     * @return mixed|null
     * @throws Mage_Core_Exception
     * @throws Zend_Http_Client_Exception
     * @throws Zend_Json_Exception
     */
    public function request($url, $options, $method = Zend_Http_Client::POST)
    {
        $defaultOptions = array(
            'client_id'		=>	$this->_config->getClientId(),
            'client_secret'	=>	$this->_config->getClientSecret(),
            'redirect_uri'	=>	'',
            'code'			=>	'SOME_CODE',
        );
        $options = array_merge($defaultOptions, $options);
        $options = $this->_prepareParams($options);
        return $this->_oauthRequest($url, $options, $method);
    }

    /**
     * @param $method
     * @param $url
     * @param $options
     * @param string $tokenScope
     * @return Zend_Http_Response
     * @throws Mage_Core_Exception
     * @throws Zend_Http_Client_Exception
     */
    public function authenticatedRequest($method, $url, $options, $tokenScope = 'WEDEC_VALIDATE_ADDRESS')
    {

        $this->_tokenScope = $tokenScope;
        $accessToken = $this->getAccessToken();
        $client = $this->_getHttpClient()
            ->resetParameters()
            ->setUri($url)
            ->setHeaders(['Authorization'=>'Bearer ' . $accessToken])
            ->setMethod($method);

        if($method === Zend_Http_Client::POST ) {
            $optionsJson = json_encode($options);
            $client->setRawData($optionsJson, 'application/json');
        } else {
            $client->setParameterGet($options);
        }

        try {
            /** @var Zend_Http_Response $response */
            $response = $client->request();
        } catch(Exception $e) {
            throw new Mage_Core_Exception('Service temporarily unavailable.');
        }
        if($response->getStatus() == 401 && $this->requestCounter < 1 ) {
            $this->requestCounter++;
            $this->_createFreshToken();
            return $this->authenticatedRequest($method, $url, $options, $tokenScope);
        }
        $this->_config->debugLog($url, $options, $response);
        return $response;
    }

    /**
     * @return string
     */
    public function getTokenScope()
    {
        return $this->_tokenScope;
    }
    public function setTokenScope(string $tokenScope)
    {
        $this->_tokenScope = $tokenScope;
        return $this;
    }
}
