<?php

class Insign_Swisspost_Adminhtml_BarcodeController extends Mage_Adminhtml_Controller_Action
{
    public function indexAction()
    {
        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock('insign_swisspost/adminhtml_barcode'));
        $this->getLayout()->getBlock('head')->setTitle('Swiss Post Barcode');
        $this->renderLayout();
    }

    public function createbarcodeAction()
    {
        if(!$this->getRequest()->getParam('shipment_id')) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('insign_swisspost')->__('Shipment is not selected')
            );
            $this->_redirect('adminhtml/sales_shipment');
        }

        $this->loadLayout();
        $this->_addContent($this->getLayout()->createBlock('insign_swisspost/adminhtml_barcode_edit'));
        $this->getLayout()->getBlock('head')->setTitle('Swiss Post Barcode generation');
        $this->renderLayout();
    }
    /**
     * Print label for one specific shipment
     *
     */
    public function printBarcodeAction()
    {
        try {
            $shipment = $this->_initShipment();
            $labelContent = $shipment->getShippingLabel();
            if ($labelContent) {
                $pdfContent = null;
                if (stripos($labelContent, '%PDF-') !== false) {
                    $pdfContent = $labelContent;
                } else {
                    $pdf = new Zend_Pdf();
                    $page = $this->_createPdfPageFromImageString($labelContent);
                    if (!$page) {
                        $this->_getSession()->addError(Mage::helper('sales')->__('File extension not known or unsupported type in the following shipment: %s', $shipment->getIncrementId()));
                    }
                    $pdf->pages[] = $page;
                    $pdfContent = $pdf->render();
                }

                return $this->_prepareDownloadResponse(
                    'ShippingLabel(' . $shipment->getIncrementId() . ').pdf',
                    $pdfContent,
                    'application/pdf'
                );
            }
        } catch (Mage_Core_Exception $e) {
            $this->_getSession()->addError($e->getMessage());
        } catch (Exception $e) {
            Mage::logException($e);
            $this->_getSession()
                ->addError(Mage::helper('sales')->__('An error occurred while creating shipping label.'));
        }
        $this->_redirect('*/sales_order_shipment/view', array(
            'shipment_id' => $this->getRequest()->getParam('shipment_id')
        ));
    }
    private function _arrayCleaner($input) {
        foreach ($input as &$value) {
            if (is_array($value)) {
                $value = $this->_arrayCleaner($value);
            }
        }

        return array_filter($input, function($item){
            return $item !== null && $item !== '' ;
        });
    }
    public function generateAction()
    {
        $shipmentId = $this->getRequest()->getParam('shipment_id');
        $shipment = Mage::getModel('sales/order_shipment')->load($shipmentId);
        if ($data = $this->getRequest()->getPost('barcode')) {
            $data = $this->_arrayCleaner($data);
            $serviceCode = $data['service_code'] ?? '';
            $serviceCode = array_map('trim',(explode(',', $serviceCode)));
            $additionalService = $data['additional_service'] ?? null;
            if($additionalService && !is_array($additionalService)) {
                $serviceCode = array_merge($serviceCode, $additionalService);
                unset($data['additional_service']);
            }
            $data['item']['attributes']['przl'] = $serviceCode;
            unset($data['service_code']);
            try {
                Mage::getModel('insign_swisspost/barcodeProvider')->createBarcode($shipment, $data);
                Mage::getSingleton('adminhtml/session')->addSuccess(
                    Mage::helper('insign_swisspost')->__('Barcode is successfully generated')
                );
                $this->_redirect('adminhtml/sales_shipment/view', array('shipment_id' => $this->getRequest()->getParam('shipment_id')));
            } catch (Mage_Core_Exception $e) {
                Mage::getSingleton('adminhtml/session')->addError(
                    Mage::helper('insign_swisspost')->__($e->getMessage())
                );
                return $this->_redirectReferer();
            }
        }
    }
    public function printAction()
    {
        $id = (int) $this->getRequest()->getParam('id');
        if(!$id) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('insign_swisspost')->__('No Items are selected')
            );
            return $this->_redirectReferer();
        }
        $barcode = Mage::getModel('insign_swisspost/barcode')->load($id);
        $config = Mage::getModel('insign_swisspost/config');
        $file = $barcode->getLabelFilePath();
        $file = $config->getFullBarcodePathByFileName($file);
        $contentType = mime_content_type($file);
        $contentType = $contentType ? $contentType :'application/octet-stream';
        $fileInfo = pathinfo($file);

        if(!$file) {
            Mage::logException('Barcode does not exists');
            return $this->_redirectReferer();
        }
        header('Content-type: '.$contentType);
        header('Content-Disposition: inline; filename="' . $barcode->getIndentyCode().'.'.$fileInfo['extension'] . '"');
        header('Content-Transfer-Encoding: binary');
        header('Accept-Ranges: bytes');
        // Read the file
        @readfile($file);
        exit();
    }
    public function downloadAction()
    {
        $id = (int) $this->getRequest()->getParam('id');
        if(!$id) {
            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('insign_swisspost')->__('No Items are selected')
            );
            return $this->_redirectReferer();
        }
        $barcode = Mage::getModel('insign_swisspost/barcode')->load($id);
        $config = Mage::getModel('insign_swisspost/config');
        $file = $barcode->getLabelFilePath();
        $filepath = $config->getFullBarcodePathByFileName($file);
        $fileInfo = pathinfo($filepath);
        $fileName = $barcode->getIdentityCode().'.'.$fileInfo['extension'];
        if ($filepath) {
            try {
                $this->_prepareDownloadResponse( $fileName, array('type' => 'filename', 'value' => $filepath));

            } catch (Exception $e) {
                $this->_getSession()->addError($e->getMessage());
            }
        } else {
            $this->_getSession()->addError($filepath . ' does not found');
            return $this->_redirectReferer();
        }
    }
    public function massPrintAction()
    {
        $ids = $this->getRequest()->getParam('barcode_id');
        if(!$ids || empty($ids)) {

            Mage::getSingleton('adminhtml/session')->addError(
                Mage::helper('insign_swisspost')->__('No Items are selected')
            );
            return $this->_redirectReferer();
        }
        return $this->_redirectReferer();
    }
}
