;(function($){
    $.fn.extend({
        donetyping: function(callback,timeout){
            timeout = timeout || 1e3; // 2 second default timeout
            var timeoutReference,
                doneTyping = function(el){
                    if (!timeoutReference) return;
                    timeoutReference = null;
                    callback.call(el);
                };
            return this.each(function(i,el){
                var $el = $(el);
                $el.is(':input') && $el.on('keyup keypress paste',function(e){
                    if (e.type=='keyup' && e.keyCode!=8) return;

                    if (timeoutReference) clearTimeout(timeoutReference);
                    timeoutReference = setTimeout(function(){
                        doneTyping(el);
                    }, timeout);
                }).on('blur',function(){
                    doneTyping(el);
                });
            });
        },
    });
})(jQuery);

let messageTemplate = function (status, quality, ele) {
    let formId = ele.closest('form').attr('id');
    let formtype = (formId === 'co-billing-form') ? 'billing': 'shipping';
    let liElement = '<ul class="messages" id="validator-FORMID"><li class="SWISSPSOT_QUALITY">' +
        '<input type="hidden" name="'+formtype+'[post_address_status]" value="'+quality+'" title="" id="'+formtype+':post_address_status" class="input-text" />'+
        '<ul><li><span id="validate-message">SWISSPOST_MESSAGE</span></li></ul></li></ul>';
    let msg = Translator.translate('The delivery address is valid but could not be verified. Please check that the delivery address is correct and then continue.');
    let spanClass = 'yellow notice-msg';
    if(status === 'ok') {
        if(['CERTIFIED', 'DOMICILE_CERTIFIED'].includes(quality)){
            spanClass = 'green info success-msg';
            msg = Translator.translate('The delivery address was successfully validated.');
        } else if(['VALID', 'USABLE', 'FIXED'].includes(quality)) {
            spanClass = 'blue info note-msg';
            msg = Translator.translate('The delivery address is valid but could not be verified. Please check that the delivery address is correct and then continue.');
        } else if(['UNUSABLE', 'COMPROMISED'].includes(quality)) {
            spanClass = 'yellow notice-msg';
            msg = Translator.translate('The validation of the delivery address has shown that it is invalid. Please check the correctness of the delivery address before you continue.');
        }
    } else {
        spanClass = 'yellow warning notice-msg';
        msg = Translator.translate('An error occurred during the validation of the delivery address');
    }
    liElement = liElement.replace('FORMID', formId).replace('SWISSPSOT_QUALITY', spanClass).replace('SWISSPOST_MESSAGE', msg);
    let eleAddressValidator = jQuery('#validator-'+formId);
    if(eleAddressValidator.length > 0) {
        eleAddressValidator.replaceWith(liElement);
    } else {
        let buttonSet = ele.closest('form').find('#'+formtype+'-buttons-container');
        jQuery(liElement).insertBefore(buttonSet);
    }
};

jQuery(function() {
    if(!isSwisspostEnabled) {
        return false;
    }
    let isSwitzerland = function (formType) {
        if(jQuery('#'+formType+'\\:country_id').val() !== 'CH') {
            //console.log("Only switzerland is allowed");
            let formId = (formType === 'billing') ? 'co-billing-form': 'co-shipping-form';
            let messageUl = '#validator-'+formId;
            if(jQuery(messageUl).length > 0) {
                jQuery(messageUl).remove();
            }
            return false;
        }
        return true;
    };
    let checkoutFormType = ['billing', 'shipping'];
    /** start validation **/

    let validationInputElements = ['firstname', 'lastname', 'street1', 'postcode', 'city', 'country_id'];
    jQuery.each(checkoutFormType, function( index, formType ) {
        jQuery.each(validationInputElements, function( index, elementType ) {
            let ele = jQuery('#'+formType+'\\:'+elementType);
            if(jQuery(ele).is('select')) {
                jQuery(ele).on('change', function () {
                    makeValidationRequest(ele);
                    if(isSwitzerland(formType)){
                        autocomplete();
                    }
                })
            } else {
                jQuery(ele).donetyping(function () {
                    makeValidationRequest(ele);
                });
            }
        });
    });
    let validate = function (formType) {
        if(!isSwisspostAddressValidationEnabled) { // check if country is Switzerland & validation Enabled
            //console.log("module is not enabled");
            return false;
        }
        if(!isSwitzerland(formType)) {
            return false;
        }
        let valid = true;
        validationInputElements.forEach(function (elementType, ind) {
            let item = jQuery('#' + formType + '\\:' + elementType);
            if(!item.length || item.val() === '') {
                //console.log("One of the item is not valid");
                valid = false
                return false;
            }
        });
        return valid;
    };
    let makeRequestCounter = 0;
    let makeValidationRequest = function (ele) { // make ajax request and validate
        let formId = ele.closest('form').attr('id');
        let formType = (formId === 'co-billing-form') ? 'billing' : 'shipping';
        if(!validate(formType)) {
            makeRequestCounter = 0;
            return false;
        }
        makeRequestCounter = 1;
        let requestData = jQuery('form#'+formId).serializeArray();
        jQuery.ajax({
            type: "POST",
            data: requestData,
            url: swisspostValidationUrl,
            success: function (result) {
                messageTemplate(result.status, result.quality, ele);
            }
        }).done(function() {
            makeRequestCounter = 0;
        });
    };

    /** end of validation **/

    /** start of autocomplete **/

    let zipCityStreetAutoComplete = function(formType, elementType) {
        let ele = jQuery('#'+formType+'\\:'+elementType);
        if(ele.length <= 0) {
            return false;
        }
        let autocompleteEvent = function (event, ui) {
            event.preventDefault();
            if(!isSwitzerland(formType)) {
                return;
            }

            if(elementType === 'street1') {
                jQuery(ele).val(ui.item.street);
            } else {
                jQuery('#'+formType+'\\:postcode').val(ui.item.zip);
                jQuery('#'+formType+'\\:city').val(ui.item.city);
            }
        };
        jQuery(ele).autocomplete({ //autocomplete start here
            source: function( request, response ) {
                if(!isSwitzerland(formType)) {
                    return;
                }
                let urlPath = swisspostAutocompleteZipUrl;
                let dataToSend = {
                    zipCity: ele.val()
                };
                if(elementType === 'street1') { //if street
                    dataToSend = {
                        streetname : jQuery(ele).val(),
                        zip : jQuery('#'+formType+'\\:postcode').val()
                    };
                    urlPath = swisspostAutocompleteStreetUrl;
                }
                jQuery.ajax({
                    url: urlPath,
                    data: dataToSend,
                    success: function( data ) {
                        response(data);
                    }
                });
            },
            minLength: 2,
            open: function (e, ui) {
                let acData = jQuery(this).data('ui-autocomplete');
                acData
                    .menu
                    .element
                    .find('li')
                    .each(function () { //make bold
                        let me = jQuery(this);
                        let keywords = acData.term.split(' ').join('|');
                        let textWrapper = me.find('.ui-menu-item-wrapper');
                        let text = textWrapper.text();
                        let newTextHtml = text.replace(new RegExp("(" + keywords + ")", "gi"), '<b>$1</b>');
                        textWrapper.html(newTextHtml);
                    });
            },
            select: function( event, ui ) {
                autocompleteEvent(event, ui);
                makeValidationRequest(ele);
            },
            focus: function(event, ui) {
                autocompleteEvent(event, ui);
            }
        });
    };

    let autocompleteFields = ['city', 'postcode', 'street1'];
    let autocomplete = function () {
        if(!isSwisspostAutocompleteEnabled) {
            return false;
        }
        jQuery.each(checkoutFormType, function( index, formType ) {
            jQuery.each(autocompleteFields, function( index, elementType ) {
                zipCityStreetAutoComplete(formType, elementType);
            });
        });
    };
    autocomplete();
});

jQuery(document).ready(function() { // loggedIn user's address list
    let checkAddressQuality = function (addressId, ele) {
        jQuery.ajax({
            url: swisspostCheckAddressQuality,
            data: {id:addressId},
            success: function(data) {
                if(data.status ==='ok') {
                    messageTemplate(data.status, data.quality, ele);
                }
            }
        });
    };
    let getQuality = function(ele) {
        if(ele.val() !== '') {
            checkAddressQuality(ele.val(), ele)
        }
    };
    /** billing Address **/
    let billingSelected = jQuery('#billing-address-select');
    if(billingSelected.length > 0) {
        billingSelected.on('change', function () {
            getQuality(jQuery(this))
        });
        getQuality(billingSelected)
    }
    /** shipping Address **/
    let shippingSelected = jQuery('#shipping-address-select');
    if(shippingSelected.length > 0) {
        shippingSelected.on('change', function () {
            getQuality(jQuery(this))
        });
        getQuality(shippingSelected)
    }
});
